#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
    char version[50];
    char date[50];
} VersionInfo;

typedef struct {
    char title[100];
    char author[100];
    VersionInfo versionInfo;
    struct Book* bibliography[10];
    int bibliographyCount;
} Book;

typedef struct {
    Book* books[100];
    int bookCount;
} Catalog;

void addBibliography(Book* book, Book* bibliography) {
    book->bibliography[book->bibliographyCount++] = bibliography;
}

void addBook(Catalog* catalog, Book* book) {
    catalog->books[catalog->bookCount++] = book;
}

void displayAllBooks(Catalog* catalog) {
    const char* memo = "all";
    showBooks(catalog->books, catalog->bookCount, memo);
}

void searchBooksByTitle(Catalog* catalog, const char* searchTitle, Book*** foundBooks, int* foundCount) {
    /*
    foundBooks (pointer to a pointer to Book), foundCount (pointer to an int) 
    The function adds the matching books to the foundBooks array. The foundCount is incremented accordingly.
    After this function returns, the foundBooks and foundCount values are accessible in the main function. 
    */
    *foundBooks = (Book**)malloc(sizeof(Book*) * catalog->bookCount);
    *foundCount = 0;

    for (int i = 0; i < catalog->bookCount; i++) {
        if (strcasecmp(catalog->books[i]->title, searchTitle) == 0) {
            (*foundBooks)[(*foundCount)++] = catalog->books[i];
        }
    }

    *foundBooks = (Book**)realloc(*foundBooks, sizeof(Book*) * (*foundCount));
}

void addNewVersion(Catalog* catalog, const char* title, VersionInfo newVersion) {
        /* 
        default setting: 1) the author and bibliography have no change in different versions.
                         2) Book titles are not case sensitive.
                         3) duplicate titles of books are not considered.

        check if the book exists in catalog,
        if exists, add a new version of the book to catalog, results in 2 versions of a book (a exist one and a newly added one)
        if not, printf("Failed to add new version. No book found with the given title.\n")
        */

    // TODO IMPLEMENT 
   
}

void showBooks(Book** bookList, int bookCount, const char* searchTitle) {
    if (strcmp(searchTitle, "all") != 0) {
        if (sizeof(bookList) == 0 ) {
            printf("No books found with the given title.\n\n");
        } else {
            printf("==== Books matching the title '%s': ====\n", searchTitle);
        }
    } else {
        printf("==== All books in the catalog: ====\n");
    }

    for (int i = 0; i < bookCount; i++) {
        Book* book = bookList[i];
        printf("Title: %s\n", book->title);
        printf("Author: %s\n", book->author);
        printf("Version: %s\n", book->versionInfo.version);
        printf("Date: %s\n", book->versionInfo.date);
        printf("Bibliography:\n");
        for (int j = 0; j < book->bibliographyCount; j++) {
            Book* biblioBook = book->bibliography[j];
            printf("- %d Title: %s\n", j + 1, biblioBook->title);
            printf("  Author: %s\n", biblioBook->author);
            printf("  Version: %s\n", biblioBook->versionInfo.version);
            printf("  Date: %s\n", biblioBook->versionInfo.date);
        }
        printf("\n");
    }
}


int main() {
    Catalog catalog;
    catalog.bookCount = 0;

    // Create books
    VersionInfo program_paradigms_version = {"First Edition", "12-2010"};
    Book program_paradigms = {"Programming Paradigms", "Maurizio Gabbrielli", program_paradigms_version, {NULL}, 0};

    VersionInfo programming_languages_version = {"Fourth Edition", "06-2008"};
    Book programming_languages = {"Programming languages", "Sarah Guido", programming_languages_version, {NULL}, 0};

    VersionInfo advanced_intelligent_paradigms_version = {"Second Edition", "05-1995"};
    Book advanced_intelligent_paradigms = {"Advanced Intelligent Paradigms", "Simone Martini", advanced_intelligent_paradigms_version, {NULL}, 0};

    // Add bibliography books
    addBibliography(&program_paradigms, &programming_languages);
    addBibliography(&program_paradigms, &advanced_intelligent_paradigms);

    // Create library catalog
    addBook(&catalog, &program_paradigms);

    // Display all books in the catalog
    displayAllBooks(&catalog);

    // Comment the following code and run the file directly, the results may be helpful for understanding the code.
    // Add a new version to an existing book
    VersionInfo new_version = {"Second Edition", "06-2013"};
    addNewVersion(&catalog, "Programming Paradigms", new_version);

    // Search books by title
    Book** foundBooks;
    int foundCount;
    const char* searchTitle = "Programming Paradigms";
    searchBooksByTitle(&catalog, searchTitle, &foundBooks, &foundCount);
    showBooks(foundBooks, foundCount, searchTitle);

    return 0;
}
