#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <assert.h>

typedef struct {
    char version[50];
    char date[50];
} VersionInfo;

typedef struct {
    char title[100];
    char author[100];
    VersionInfo versionInfo;
    struct Book* bibliography[10];
    int bibliographyCount;
} Book;

typedef struct {
    Book* books[100];
    int bookCount;
} Catalog;

void addBibliography(Book* book, Book* bibliography) {
    book->bibliography[book->bibliographyCount++] = bibliography;
}

void addBook(Catalog* catalog, Book* book) {
    catalog->books[catalog->bookCount++] = book;
}

void displayAllBooks(Catalog* catalog) {
    const char* memo = "all";
    showBooks(catalog->books, catalog->bookCount, memo);
}

void searchBooksByTitle(Catalog* catalog, const char* searchTitle, Book*** foundBooks, int* foundCount) {
    *foundBooks = (Book**)malloc(sizeof(Book*) * catalog->bookCount);
    *foundCount = 0;

    for (int i = 0; i < catalog->bookCount; i++) {
        if (strcasecmp(catalog->books[i]->title, searchTitle) == 0) {
            (*foundBooks)[(*foundCount)++] = catalog->books[i];
        }
    }

    *foundBooks = (Book**)realloc(*foundBooks, sizeof(Book*) * (*foundCount));
}

void addNewVersion(Catalog* catalog, const char* title, VersionInfo newVersion) {
    // if you want to run these test cases, easily paste your addNewVersion here.
}

void showBooks(Book** bookList, int bookCount, const char* searchTitle) {
    if (strcmp(searchTitle, "all") != 0) {
        if (sizeof(bookList) == 0 ) {
            printf("No books found with the given title.\n\n");
        } else {
            printf("==== Books matching the title '%s': ====\n", searchTitle);
        }
    } else {
        printf("==== All books in the catalog: ====\n");
    }

    for (int i = 0; i < bookCount; i++) {
        Book* book = bookList[i];
        printf("Title: %s\n", book->title);
        printf("Author: %s\n", book->author);
        printf("Version: %s\n", book->versionInfo.version);
        printf("Date: %s\n", book->versionInfo.date);
        printf("Bibliography:\n");
        for (int j = 0; j < book->bibliographyCount; j++) {
            Book* biblioBook = book->bibliography[j];
            printf("- %d Title: %s\n", j + 1, biblioBook->title);
            printf("  Author: %s\n", biblioBook->author);
            printf("  Version: %s\n", biblioBook->versionInfo.version);
            printf("  Date: %s\n", biblioBook->versionInfo.date);
        }
        printf("\n");
    }
}

// Define the test cases
void test_addNewVersion_existingTitle() {
    Catalog catalog;
    catalog.bookCount = 0;

    // Create books
    VersionInfo versionInfo = {"First Edition", "01-2012"};
    Book databaseSystems = {"Database Systems", "Author 1", versionInfo, {NULL}, 0};

    // Add the book to the catalog
    addBook(&catalog, &databaseSystems);

    // Create the new version
    VersionInfo newVersionInfo = {"Second Edition", "06-2013"};

    // Add the new version to the existing book
    addNewVersion(&catalog, "Database Systems", newVersionInfo);

    // Check if the new version was added successfully
    bool foundNewVersion = false;
    for (int i = 0; i < catalog.bookCount; i++) {
        Book* book = catalog.books[i];

        if (strcasecmp(book->title, "Database Systems") == 0) {
            // Check if the new version exists
            if (strcasecmp(book->versionInfo.version, newVersionInfo.version) == 0){
                foundNewVersion = true;
            }
        }
    }
    // Assert that the new version was added successfully
    assert(foundNewVersion);

}

void test_addNewVersion_nonExistingTitle() {
    Catalog catalog;
    catalog.bookCount = 0;

    // Create books
    VersionInfo programming_languages_version = {"Fourth Edition", "06-2008"};
    Book programming_languages = {"Programming languages", "Sarah Guido", programming_languages_version, {NULL}, 0};

    // Create library catalog
    addBook(&catalog, &programming_languages);

    // Create the new version
    VersionInfo newVersionInfo = {"Second Edition", "06-2013"};

    // Add the new version with a non-existing title
    addNewVersion(&catalog, "Introduction to Programming", newVersionInfo);

    // Check if the new version was failed to added
    bool foundNewVersion = false;
    for (int i = 0; i < catalog.bookCount; i++) {
        Book* book = catalog.books[i];
        if (strcasecmp(book->title, "Introduction to Programming") == 0) {
            // Check if the new version exists
            if (strcasecmp(book->versionInfo.version, newVersionInfo.version) == 0 &&
                strcasecmp(book->versionInfo.date, "06-2013") == 0) {
                foundNewVersion = true;
            }
        }
    }
    // Assert that the new version was failed to added
    assert(!foundNewVersion);
}

int main() {
    // Run the test cases
    // remember to paste your addNewVersion function above.
    test_addNewVersion_existingTitle();
    test_addNewVersion_nonExistingTitle();

    printf("All tests passed successfully.\n");

    return 0;
}
