package pl_java.src;

import java.util.ArrayList;
import java.util.List;


class VersionInfo {
    String version;
    String date;

    public VersionInfo(String version, String date) {
        this.version = version;
        this.date = date;
    }
}

class Book {
    String title;
    String author;
    VersionInfo versionInfo;
    List<Book> bibliography;

    public Book(String title, String author, VersionInfo versionInfo) {
        this.title = title;
        this.author = author;
        this.versionInfo = versionInfo;
        this.bibliography = new ArrayList<>();
    }

    public void addBibliography(Book book) {
        bibliography.add(book);
    }
}

class Catalog {
    List<Book> books;

    public Catalog() {
        this.books = new ArrayList<>();
    }

    public void addBook(Book book) {
        books.add(book);
    }

    public void displayAllBooks() {
        showBooks(books, "all");
    }

    public List<Book> searchBooksByTitle(String title) {
        List<Book> foundBooks = new ArrayList<>();
        for (Book book : books) {
            if (book.title.equalsIgnoreCase(title)){
                foundBooks.add(book);
            }
        }
        return foundBooks;
    }

    public void addNewVersion(String title, VersionInfo newVersion) {
        /* 
        default setting: 1) the author and bibliography have no change in different versions.
                         2) Book titles are not case sensitive.
                         3) duplicate titles of books are not considered.

        check if the book exists in catalog,
        if exists, add a new version of the book to catalog, results in 2 versions of a book (a exist one and a newly added one)
        if not, System.out.println("Failed to add new version. No book found with the given title.\n")
        */

        // TODO IMPLEMENT
        

    }

    public void showBooks(List<Book> bookList, String searchTitle) {
        if (!searchTitle.equals("all")) {
            if (bookList.isEmpty()) {
                System.out.println("No books found with the given title.\n");
            } else {
                System.out.printf("==== Books matching the title '%s': ====%n", searchTitle);
            }
        } else {
            System.out.println("==== All books in the catalog: ====");
        }

        for (Book book : bookList) {
            System.out.println("Title: " + book.title);
            System.out.println("Author: " + book.author);
            System.out.println("Version: " + book.versionInfo.version);
            System.out.println("Date: " + book.versionInfo.date);
            System.out.println("Bibliography:");

            List<Book> bibliography = book.bibliography;
            for (int i = 0; i < bibliography.size(); i++) {
                Book biblioBook = bibliography.get(i);
                System.out.println("- " + i + " Title: " + biblioBook.title);
                System.out.println("  Author: " + biblioBook.author);
                System.out.println("  Version: " + biblioBook.versionInfo.version);
                System.out.println("  Date: " + biblioBook.versionInfo.date);
            }
            System.out.println();
        }
    }
}

public class LibraryCatalog { 

    public static void main(String[] args) {
        Catalog catalog = new Catalog();
 
        // Create books
        Book programParadigms = new Book("Programming Paradigms", "Maurizio Gabbrielli", new VersionInfo("First Edition", "12-2010"));
        Book programmingLanguages = new Book("Programming languages", "Sarah Guido", new VersionInfo("Fourth Edition", "06-2008"));
        Book AdvancedIntelligentParadigms = new Book("Advanced Intelligent Paradigms", "Simone Martini", new VersionInfo("Second Edition", "05-1995"));

        // Add bibliography books
        programParadigms.addBibliography(programmingLanguages);
        programParadigms.addBibliography(AdvancedIntelligentParadigms);

        // Create library catalog
        catalog.addBook(programParadigms);

        // Display all books in the catalog
        catalog.displayAllBooks();

        // Comment the following code and run the file directly, the results may be helpful for understanding the code.
        // Add a new version to a existing book
        catalog.addNewVersion("Programming Paradigms", new VersionInfo("Second Edition", "06-2013"));
        
        // Search books by title
        String searchTitle = "Programming Paradigms";
        List<Book> foundBooks = catalog.searchBooksByTitle(searchTitle);
        catalog.showBooks(foundBooks, searchTitle);
    }
}
