package pl_java.src;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

import java.util.List;

public class LibraryCatalogTest {
    private Catalog catalog;

    @Before
    public void setUp() {
        catalog = new Catalog();
    }

    @Test
    public void testAddNewVersion() {
        // Add a book to the catalog
        Book book = new Book("Java Fundamentals", "John Smith", new VersionInfo("First Edition", "01-2020"));
        catalog.addBook(book);

        // Create a new version
        VersionInfo newVersion = new VersionInfo("Second Edition", "06-2021");

        // Add the new version to the book
        catalog.addNewVersion("Java Fundamentals", newVersion);

        // Verify that the new version is added
        assertEquals(2, catalog.books.size());
        Book originalBook = catalog.books.get(0);
        Book updatedBook = catalog.books.get(1); // Get the updated book
        assertEquals("Java Fundamentals", updatedBook.title);
        assertEquals("John Smith", updatedBook.author);
        assertEquals("Second Edition", updatedBook.versionInfo.version);
        assertEquals("06-2021", updatedBook.versionInfo.date);
        assertNotEquals(originalBook.versionInfo.version, updatedBook.versionInfo.version);
    }

    @Test
    public void testAddNewVersion2() {
        // Add a book to the catalog
        Book book1 = new Book("Introduction to Java", "John Smith", new VersionInfo("First Edition", "01-2020"));
        Book book2 = new Book("Advanced Java Programming", "Jane Doe", new VersionInfo("Second Edition", "06-2021"));
        catalog.addBook(book1);
        catalog.addBook(book2);

        // Create a new version
        VersionInfo newVersion = new VersionInfo("Third Edition", "06-2023");

        // Add the new version to the book
        catalog.addNewVersion("Advanced Java Programming", newVersion);

        // Verify that the new version is added
        assertEquals(3, catalog.books.size());
        List<Book> foundBooks = catalog.searchBooksByTitle("Advanced Java Programming");
        assertEquals(2, foundBooks.size());
        Book version1 = foundBooks.get(0);
        Book version2 = foundBooks.get(1);
        assertEquals(version1.author, version2.author);
        assertNotEquals(version1.versionInfo.version, version2.versionInfo.version);
    }
}


