from LibraryCatalog import Book, VersionInfo, Catalog

def testAddBook():
    version = "First Edition"
    date = "02-2022"
    versionInfo = VersionInfo(version, date)

    title = "Introduction to Python3"
    author = "John Doe"
    book = Book(title, author, versionInfo)

    assert book.title == title
    assert book.author == author
    assert book.versionInfo == versionInfo
    assert book.bibliography == []

    biblioVersion = "Second Edition"
    biblioDate = "05-2014"
    biblioVersionInfo = VersionInfo(biblioVersion, biblioDate)

    biblioBook = Book("Learn Python", "Jane Smith", biblioVersionInfo)
    book.addBibliography(biblioBook)

    assert len(book.bibliography) == 1
    assert book.bibliography[0] == biblioBook


def testCatalog():
    catalog = Catalog()

    assert catalog.books == []

    title = "Data Structures and Algorithms"
    author = "John Doe"
    version = "Second Edition"
    date = "03-2018"
    versionInfo = VersionInfo(version, date)
    book = Book(title, author, versionInfo)

    catalog.addBook(book)

    assert len(catalog.books) == 1
    assert catalog.books[0] == book

    searchTitle = title
    foundBooks = catalog.searchBooksByTitle(searchTitle)

    assert len(foundBooks) == 1
    assert foundBooks[0].title == title
    assert foundBooks[0].author == author
    assert foundBooks[0].versionInfo.version == version
    assert foundBooks[0].versionInfo.date == date

    newVersion = "Third Edition"
    newDate = "12-2017"
    newVersionInfo = VersionInfo(newVersion, newDate)
    catalog.addNewVersion(title, newVersionInfo)

    assert len(catalog.books) == 2
    assert catalog.books[1].title == title
    assert catalog.books[1].author == author
    assert catalog.books[1].versionInfo.version == newVersion
    assert catalog.books[1].versionInfo.date == newDate

    assert foundBooks[0].versionInfo.version != catalog.books[1].versionInfo.version


if __name__ == "__main__":
    testAddBook()
    testCatalog()
